library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity AES_128_enkripcija is
	Port 
	( 
		clk		 	 			: in   STD_LOGIC;
		start			 			: in   STD_LOGIC;
		kraj			 			: in   STD_LOGIC;
		kljuc_in     			: in	 STD_LOGIC_VECTOR (127 downto 0);
		data_in	 				: in   STD_LOGIC_VECTOR (127 downto 0);
		izvorisna_adresa		: in   STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa		: in   STD_LOGIC_VECTOR (31 downto 0);
		izvorisna_adresa_out	: out  STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa_out	: out  STD_LOGIC_VECTOR (31 downto 0);
		start_out	 			: out  STD_LOGIC;
		kraj_out		 			: out  STD_LOGIC;
		data_out  	 			: out  STD_LOGIC_VECTOR (127 downto 0)
	);
end AES_128_enkripcija;

architecture shema of AES_128_enkripcija is

component razvijanje_kljuca is
	Port 
	( 
		clk 			: in  STD_LOGIC;
		kljuc 		: in  STD_LOGIC_VECTOR (127 downto 0);
      kljuc0_out 	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc1_out	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc2_out 	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc3_out 	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc4_out 	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc5_out 	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc6_out 	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc7_out 	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc8_out 	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc9_out 	: out STD_LOGIC_VECTOR (127 downto 0);
      kljuc10_out : out STD_LOGIC_VECTOR (127 downto 0)
	);
	
end component;

component data_enkripcija is
    Port 
	( 
		clk 			 			: in    STD_LOGIC;
		start			 			: in    STD_LOGIC;
		kraj						: in    STD_LOGIC;
		izvorisna_adresa		: in    STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa		: in    STD_LOGIC_VECTOR (31 downto 0);
		data_in	 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc0_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc1_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc2_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc3_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc4_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc5_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc6_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc7_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc8_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc9_in 	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		kljuc10_in	 			: in    STD_LOGIC_VECTOR (127 downto 0);
		data_out		 			: out   STD_LOGIC_VECTOR (127 downto 0);
		izvorisna_adresa_out	: out   STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa_out	: out   STD_LOGIC_VECTOR (31 downto 0);
		start_out	 			: out	  STD_LOGIC;
		kraj_out		 			: out	  STD_LOGIC
	);
end component;

-- kljucevi za potrebe povezivanja entiteta data_enkripcija i razvijanje kljuca
signal kljuc0_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc1_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc2_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc3_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc4_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc5_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc6_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc7_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc8_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc9_int:std_logic_vector(127 downto 0):= (others => '0');
signal kljuc10_int:std_logic_vector(127 downto 0):= (others => '0');


begin

	kljuc: razvijanje_kljuca
	port map
	(
		clk => clk,
		kljuc => kljuc_in,
		kljuc0_out => kljuc0_int,
		kljuc1_out => kljuc1_int,
		kljuc2_out => kljuc2_int,
		kljuc3_out => kljuc3_int,
		kljuc4_out => kljuc4_int,
		kljuc5_out => kljuc5_int,
		kljuc6_out => kljuc6_int,
		kljuc7_out => kljuc7_int,
		kljuc8_out => kljuc8_int,
		kljuc9_out => kljuc9_int,
		kljuc10_out => kljuc10_int
	);
	
	data: data_enkripcija
	port map
	(
		clk => clk,
		start => start,
		kraj => kraj,
		izvorisna_adresa => izvorisna_adresa,
		odredisna_adresa => odredisna_adresa,
		data_in => data_in,
		izvorisna_adresa_out => izvorisna_adresa_out,
		odredisna_adresa_out => odredisna_adresa_out,
		data_out => data_out,
		start_out => start_out,
		kraj_out => kraj_out,
		kljuc0_in => kljuc0_int,
		kljuc1_in => kljuc1_int,
		kljuc2_in => kljuc2_int,
		kljuc3_in => kljuc3_int,
		kljuc4_in => kljuc4_int,
		kljuc5_in => kljuc5_int,
		kljuc6_in => kljuc6_int,
		kljuc7_in => kljuc7_int,
		kljuc8_in => kljuc8_int,
		kljuc9_in => kljuc9_int,
		kljuc10_in => kljuc10_int
	);
	
	
end shema;

