library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;

entity Dodavanje_IP_zaglavlje is
	Port 
	( 
		clk					: in	 STD_LOGIC;
		reset					: in	 STD_LOGIC;
		start					: in   STD_LOGIC;
		kraj					: in   STD_LOGIC;
		data_in				: in   STD_LOGIC_VECTOR (127 downto 0);
		izvorisna_adresa	: in   STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa	: in   STD_LOGIC_VECTOR (31 downto 0);
		start_out			: out  STD_LOGIC;
		kraj_out				: out  STD_LOGIC;
		data_out				: out  STD_LOGIC_VECTOR (127 downto 0)
	);
end Dodavanje_IP_zaglavlje;

architecture shema of Dodavanje_IP_zaglavlje is

--process upisa
signal i:integer;
--process ispisa
signal k:integer; -- brojac koji se aktivira kada dodje poslednji pak i kojim se ulazi u pravljenje zaglavlja
signal n:integer;

component fifo_generator
	port 
	(
		clk   : IN std_logic;
		din   : IN std_logic_VECTOR(127 downto 0);
		wr_en : IN std_logic;
		rd_en : IN std_logic;
		dout  : OUT std_logic_VECTOR(127 downto 0);
		full  : OUT std_logic;
		empty : OUT std_logic
	);
end component;

component fifo_generator_brojac
	port 
	(
		clk   : IN std_logic;
		din   : IN std_logic_VECTOR(15 downto 0);
		wr_en : IN std_logic;
		rd_en : IN std_logic;
		dout  : OUT std_logic_VECTOR(15 downto 0);
		full  : OUT std_logic;
		empty : OUT std_logic
	);
end component;

component fifo_generator_adresa
	port 
	(
		clk	: IN std_logic;
		din	: IN std_logic_VECTOR(63 downto 0);
		wr_en : IN std_logic;
		rd_en : IN std_logic;
		dout	: OUT std_logic_VECTOR(63 downto 0);
		full	: OUT std_logic;
		empty	: OUT std_logic
	);
end component;

-- signali za potrebe memorije-fifo
signal pun:std_logic;
signal prazan:std_logic;
signal upis:std_logic;
signal citanje:std_logic;
signal ulaz:std_logic_vector (127 downto 0);
signal izlaz:std_logic_vector (127 downto 0);

-- signali za potrebe brojaca-fifo

signal pun_br:std_logic;
signal prazan_br:std_logic;
signal upis_br:std_logic;
signal citanje_br:std_logic;
signal ulaz_br:std_logic_vector(15 downto 0);
signal izlaz_br:std_logic_vector(15 downto 0);

-- signali za potrebe adresa-fifo

signal pun_adr:std_logic;
signal prazan_adr:std_logic;
signal upis_adr:std_logic;
signal citanje_adr:std_logic;
signal ulaz_adr:std_logic_vector(63 downto 0);
signal izlaz_adr:std_logic_vector(63 downto 0);


begin

	memorija: fifo_generator
	port map
	(
		clk => clk,
		din => ulaz,
		wr_en => upis,
		rd_en => citanje,
		dout => izlaz,
		full => pun,
		empty => prazan
	);
	
	brojac: fifo_generator_brojac
	port map
	(
		clk => clk,
		din => ulaz_br,
		wr_en => upis_br,
		rd_en => citanje_br,
		dout => izlaz_br,
		full => pun_br,
		empty => prazan_br
	);
	
	adrese: fifo_generator_adresa
	port map
	(
		clk => clk,
		din => ulaz_adr,
		wr_en => upis_adr,
		rd_en => citanje_adr,
		dout => izlaz_adr,
		full => pun_adr,
		empty => prazan_adr
	);
	
	process ( clk, data_in, start, kraj, reset) --process citanja
		
		variable data_int:std_logic_vector (127 downto 0):= (others => '0');	
				
	begin																						
	
		data_int:=data_in;
		
		if(reset='1') then -- svi signali se postavljaju na inicijalne vrednosti
			i<=0;
			upis<='0';
			upis_br<='0';
			upis_adr<='0';
			
		else
		
			if(clk' event and clk='1') then
			
				upis<='0'; -- ako ne stigne nista upis ide na 0, isto vazi i za upis_br, zato sto oni treba da se resetuju
				upis_br<='0'; 
				upis_adr<='0';
		
				if (start='1' and kraj='0') then -- ako nam stigne samo start=1, onda se i povecava za 1 
					i<=1;
					upis<='1'; --kada dodje paket aktivira se aktivan signal u fifo memoriji upis kojm se signalizira upis u fifo memoriju
					upis_adr<='1'; -- aktivira se signal upis_adr zbog upisa adresa u fifo memroju
					ulaz<=data_int; -- podatak koji se upisuje u fifo memoriju
					ulaz_adr<=izvorisna_adresa & odredisna_adresa; -- adrese koje se upisuju u fifo memroiju adresa, vrsimo njihovo
				end if;																--spajanje
			
				if( start='0' and kraj='0') then -- u slucaju da dodje i start i kraj =0 onda proveravamo 
					if( i>=1 ) then					-- da li je i>=1 jel ako jeste to znaci da je prethodno start bio =1
						i<=i+1;
						upis<='1';
						ulaz<=data_int;
					end if;
				end if;
			
				if ( start='0' and kraj='1') then -- u slucaju da doje kraj=1 to znaci da je taj paket poslednji
					if( i>=1 ) then						-- opet proveravamo da li je i>1, to nam je potvrda je nekada starta bio =1
						i<=0;
						upis_br<='1';-- aktivria se signal upis_br, jel zelimo nesto da upisemo u fifo memoriju brojaca
						ulaz_br<=conv_std_logic_vector(i+1,16); --posto smo signal i stavili na 0, moramo ga prethodno sabrati sa 1 posto 
						upis<='1';												-- je ostao jos jedan paket, i onda se prebacuje u 16 bitni siganl
						ulaz<=data_int;
					end if;			
				end if; 
		
			
				if ( start='1' and kraj='1') then -- u slucaju da dodje samo jedan paket i sa njim i start i kraj onda
														-- onda se aktivira j signal 
					upis<='1';
					upis_br<='1';
					upis_adr<='1';
					ulaz<=data_int;
					ulaz_br<=conv_std_logic_vector(1,16);
					ulaz_adr<=izvorisna_adresa & odredisna_adresa;
				end if; 
			
			end if;--clk
		end if;--reset
	end process;


	
	process(clk,reset) --process za ispis
	
		variable zaglavlje:std_logic_vector (159 downto 0):= (others => '0');
		variable zbir:std_logic_vector (19 downto 0) := (others => '0');--za proveru checksume
		variable novi_zbir:std_logic_vector (15 downto 0):= (others => '0');--za proveru checksume
		variable total_length:integer:=0;
		
	begin
		
		if(reset='1') then -- sve porebne signale stavljamo na inicijalnu vrednsot
			n<=0;
			k<=0;
			citanje<='0';
			citanje_br<='0';
			citanje_adr<='0';
		else
			
			if(clk' event and clk='1') then
						
				if(n=0) then
					if(prazan_br='0' and prazan_adr='0') then
						citanje_br<='1'; -- citamo podatak iz fifo brojaca trebaju dva takta da prodju da bi se podatak procitao
						citanje_adr<='1'; -- isto vazi i za citanje iz fifo memroije adresa
						n<=1;
						kraj_out<='0';
					end if;
				end if;
				
				if(n=1) then
					citanje_br<='0';
					citanje_adr<='0';
					n<=2;
				end if;
			
				if(n=2) then -- slanje prvog dela zaglavlja
					total_length:=(conv_integer(izlaz_br))*128+2*128; -- dodajemo jos dva paketa zbog zaglavlja
					k<=conv_integer(izlaz_br); -- brojac kojim ispisujemo pakete, teka sada mi dobijamo vrednost koju smo procitali
																-- pre dva takta
					zaglavlje(159 downto 156):=x"4";--polje version, mi radimo sa IPv4
					zaglavlje(155 downto 152):=x"5";--polje IHL
					zaglavlje(151 downto 146):="000000";--polje DSPC
					zaglavlje(145 downto 144):="00";--polje ECN
					zaglavlje(143 downto 128):=conv_std_logic_vector(total_length,16);--polje total length
					zaglavlje(127 downto 112):=x"0000";--!!!!!!!!!!!!! polje Identification
					zaglavlje(111 downto 109):="000";--polje flags
					zaglavlje(108 downto 96):="0000000000000";--polje Fragment Offset
					zaglavlje(95 downto 88):=x"40";--polje Time To Live
					zaglavlje(87  downto 80):=x"11";-- polje Protocol, izabrao sam da protokol bude UDP
					zaglavlje(63 downto 32):=izlaz_adr(63 downto 32);
					zaglavlje(31 downto 0):=izlaz_adr(31 downto 0);
				
					--odredjivanja polja Header Checksum
					zbir:=("0000" & zaglavlje(15 downto 0))+zaglavlje(31 downto 16)+zaglavlje(47 downto 32)+zaglavlje(63 downto 48)+zaglavlje(95 downto 80)+zaglavlje(111 downto 96)+zaglavlje(127 downto 112)+zaglavlje(143 downto 128)+zaglavlje(159 downto 144);
					novi_zbir:=not ( zbir(15 downto 0)+zbir(19 downto 16) ); --sabiramo prva cetiri bita sa preostala 16 bita i vrsimo negaciju
					zaglavlje(79 downto 64):=novi_zbir;--polje Header Checksum 
				
					--pocetak slanja zaglavlja
					data_out<=zaglavlje(159 downto 32);
					start_out<='1';
					kraj_out<='0';
			
					citanje<='1'; --citamo podatak iz fifo memorije takodje nam trebaju dva takta da bi se podatak procitao
			
					n<=3;	
				end if;
		
				if(n=3) then -- slanje drugog dela zaglavlja
					data_out<=zaglavlje(31 downto 0) & x"000000000000000000000000"; -- na ovaj nacin saljemo preostala 32
																											-- bita zaglavlja gde dopunjujemo nulama
																											-- da bi dosli do podatka od 128 bita
					start_out<='0';
					kraj_out<='0';
					n<=4;
				
					if(k>=2) then -- proveramo da li je duzina paketa 2 ili nije, ako je manja od dva nema potrebe da bude aktivan signal citanje
						citanje<='1';
					else
						citanje<='0';
					end if;
									
				end if;
		
				if(n=4) then
					if(k=1) then -- ako dodje samo jedan paket
						citanje<='0';
						data_out<=izlaz;
						kraj_out<='1';
						k<=0;
						n<=0;
					else
						data_out<=izlaz;
						if(k>2) then
							k<=k-1;
							citanje<='1';
							n<=5;
						else 
							if (k=2) then
								k<=k-1;
								citanje<='0';
								n<=6;
							end if;
						end if;
					end if;		
				end if;
		
				if(n=5) then
					if(k>2) then
						data_out<=izlaz;
						citanje<='1';
						k<=k-1;
					else 
						if (k=2) then
							k<=k-1;
							data_out<=izlaz;
							citanje<='0';
							n<=6;
						end if;
					end if;
				end if;
		
				if(n=6) then
					citanje<='0';
					data_out<=izlaz;
					kraj_out<='1';
					k<=0;
					n<=0;
				end if;
		
			end if;--clk
		end if; --reset
		
	end process;
end shema;

