library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity Tx is
	Port	
	(
		clk					: in   STD_LOGIC;
		reset					: in	 STD_LOGIC;
		start_ulaz			: in   STD_LOGIC;
		kraj_ulaz			: in   STD_LOGIC;
		kljuc_ulaz			: in   STD_LOGIC_VECTOR (127 downto 0);
		ulaz					: in   STD_LOGIC_VECTOR (127 downto 0);
		izvorisna_adresa	: in   STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa	: in   STD_LOGIC_VECTOR (31 downto 0);
		izlaz					: out  STD_LOGIC_VECTOR (127 downto 0);
		start_izlaz			: out  STD_LOGIC;
		kraj_izlaz			: out  STD_LOGIC
	);
end Tx;

architecture shema of Tx is

component AES_128_enkripcija is
	Port 
	( 
		clk		 	 			: in   STD_LOGIC;
		start			 			: in   STD_LOGIC;
		kraj			 			: in   STD_LOGIC;
		kljuc_in     			: in	 STD_LOGIC_VECTOR (127 downto 0);
		data_in	 				: in   STD_LOGIC_VECTOR (127 downto 0);
		izvorisna_adresa		: in   STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa		: in   STD_LOGIC_VECTOR (31 downto 0);
		izvorisna_adresa_out	: out  STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa_out	: out  STD_LOGIC_VECTOR (31 downto 0);
		start_out	 			: out  STD_LOGIC;
		kraj_out		 			: out  STD_LOGIC;
		data_out  	 			: out  STD_LOGIC_VECTOR (127 downto 0)
	);
end component;

component Dodavanje_IP_zaglavlje is
	Port 
	( 
		clk					: in	 STD_LOGIC;
		reset					: in	 STD_LOGIC;
		start					: in   STD_LOGIC;
		kraj					: in   STD_LOGIC;
		data_in				: in   STD_LOGIC_VECTOR (127 downto 0);
		izvorisna_adresa	: in   STD_LOGIC_VECTOR (31 downto 0);
		odredisna_adresa	: in   STD_LOGIC_VECTOR (31 downto 0);
		start_out			: out  STD_LOGIC;
		kraj_out				: out  STD_LOGIC;
		data_out				: out  STD_LOGIC_VECTOR (127 downto 0)
	);
end component;

--signali koji su potreban za povezivanje komponenti AES i Dodavanje IP zaglavlja
signal start_out2, kraj_out2 :std_logic:='0';
signal data_out2:std_logic_vector (127 downto 0):=(others => '0');
signal izvorisna_adresa_out1, odredisna_adresa_out1 : std_logic_vector (31 downto 0);
begin
	
	Enrkipcija: AES_128_enkripcija -- povezivanje entiteta AES enkripcije sa odgovarajucim signalima iz bloka Tx
	port map
	(
		clk => clk,
		start => start_ulaz,
		kraj => kraj_ulaz,
		kljuc_in => kljuc_ulaz,
		data_in => ulaz,
		izvorisna_adresa => izvorisna_adresa,
		odredisna_adresa => odredisna_adresa,
		izvorisna_adresa_out => izvorisna_adresa_out1,
		odredisna_adresa_out => odredisna_adresa_out1,
		start_out => start_out2,
		kraj_out => kraj_out2,
		data_out => data_out2
	);
	
	IP_zaglavlje: Dodavanje_IP_zaglavlje -- povezivanje entiteta Dodavanje IP zaglavlja sa odgovarajucim signalima iz bloka Tx
	port map
	(
		clk => clk,
		reset => reset,
		start => start_out2,
		kraj => kraj_out2,
		data_in => data_out2,
		izvorisna_adresa => izvorisna_adresa_out1,
		odredisna_adresa => odredisna_adresa_out1,
		start_out => start_izlaz,
		kraj_out => kraj_izlaz,
		data_out => izlaz
	);
		
end shema;

